﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.IO;

using GrxCAD.ApplicationServices;
using GrxCAD.DatabaseServices;
using GrxCAD.EditorInput;
using GrxCAD.Geometry;
using GrxCAD.PlottingServices;
using GrxCAD.Runtime;

namespace PlotSample_CS_
{
	public class sample : IExtensionApplication
	{
		//初期化処理
		public void Initialize()
		{
			Editor icEd = Application.DocumentManager.MdiActiveDocument.Editor;
			icEd.WriteMessage("\nサンプルの.NETアセンブリをロードしました");
		}

		//解放処理
		public void Terminate()
		{
		}

		[CommandMethod("PLOTSAMPLE")]
		public void PlotSample()
		{
			Database icDb = Application.DocumentManager.MdiActiveDocument.Database;
			Editor icEd = Application.DocumentManager.MdiActiveDocument.Editor;

			PromptSaveFileOptions icOpt = new PromptSaveFileOptions("\n出力先を指定");
			icOpt.Filter = "PDF(*.pdf)|*.pdf";
			icOpt.InitialDirectory = System.Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory);
			PromptFileNameResult icRes = icEd.GetFileNameForSave(icOpt);
			if (icRes.Status != PromptStatus.OK) return;

			using (Transaction icTrans = icDb.TransactionManager.StartTransaction())
			{
				LayoutManager icLayMgr = LayoutManager.Current;
				Layout icLayout = icTrans.GetObject(icLayMgr.GetLayoutId(icLayMgr.CurrentLayout), OpenMode.ForWrite) as Layout;
				PlotSettingsValidator icPlSetVdr = PlotSettingsValidator.Current;
				icPlSetVdr.SetPlotType(icLayout, GrxCAD.DatabaseServices.PlotType.Extents);
				icPlSetVdr.SetUseStandardScale(icLayout, true);
				icPlSetVdr.SetStdScaleType(icLayout, StdScaleType.ScaleToFit);
				icPlSetVdr.SetPlotCentered(icLayout, true);
				icPlSetVdr.SetPlotConfigurationName(icLayout, "DWG To PDF.pc3", null);
				foreach (string mediaName in icPlSetVdr.GetCanonicalMediaNameList(icLayout))
				{
					string localName = icPlSetVdr.GetLocaleMediaName(icLayout, mediaName);
					if (localName == "ISO A4 (297.00 x 210.00 MM)")
					{
						icPlSetVdr.SetCanonicalMediaName(icLayout, mediaName);
						break;
					}
				}
				using (PlotInfo icPlInfo = new PlotInfo())
				{
					icPlInfo.Layout = icLayout.ObjectId;
					icPlInfo.OverrideSettings = icLayout;
					using (PlotInfoValidator icPlInfoVdr = new PlotInfoValidator())
					{
						icPlInfoVdr.MediaMatchingPolicy = MatchingPolicy.MatchEnabled;
						icPlInfoVdr.Validate(icPlInfo);
						using (PlotEngine icPlEng = PlotFactory.CreatePublishEngine())
						{
							using (PlotProgressDialog icPlProgDlg = new PlotProgressDialog(false, 1, true))
							{
								icPlProgDlg.set_PlotMsgString(PlotMessageIndex.CancelJobButtonMessage, "Cancel Job");
								icPlProgDlg.set_PlotMsgString(PlotMessageIndex.CancelSheetButtonMessage, "Cancel Sheet");
								icPlProgDlg.set_PlotMsgString(PlotMessageIndex.SheetSetProgressCaption, "Sheet Set Progress");
								icPlProgDlg.set_PlotMsgString(PlotMessageIndex.SheetProgressCaption, "Sheet Progress");
								icPlProgDlg.LowerPlotProgressRange = 0;
								icPlProgDlg.UpperPlotProgressRange = 100;
								icPlProgDlg.PlotProgressPos = 0;
								icPlProgDlg.OnBeginPlot();
								icPlProgDlg.IsVisible = true;
								icPlEng.BeginPlot(icPlProgDlg, null);
								icPlEng.BeginDocument(icPlInfo, icDb.OriginalFileName, null, 1, true, icRes.StringResult);
								icPlProgDlg.set_PlotMsgString(PlotMessageIndex.Status, "Plotting: " + icDb.OriginalFileName + " - " + icLayout.LayoutName);
								icPlProgDlg.OnBeginSheet();
								icPlProgDlg.LowerSheetProgressRange = 0;
								icPlProgDlg.UpperSheetProgressRange = 100;
								icPlProgDlg.SheetProgressPos = 0;
								icPlProgDlg.SheetProgressPos = 100;
								icPlProgDlg.OnEndSheet();
								icPlEng.EndDocument(null);
								icPlProgDlg.PlotProgressPos = 100;
								icPlProgDlg.OnEndPlot();
								icPlEng.EndPlot(null);
							}
						}
					}
				}
				icTrans.Commit();
			}
		}
	}
}
